﻿
Imports Newtonsoft.Json

Module Module_Blockly

    Friend LoopMillisec As Int32
    Friend SleepEveryNloops As Int32

    ' ===================================================================================================
    '  JAVASCRIPT FUNCTION CALLS
    ' ===================================================================================================
    Friend ReadOnly BlocklyFilesPath As String = Application.StartupPath + "\Blockly\Files\"
    Private ToolboxXML As String
    Friend WorkspaceXML As String

    Friend Async Sub SetBlocklyToolboxAndWorkspace()
        Await ExecuteScriptFunctionAsync("initToolboxAndWorkspace",
                                         New String() {ToolboxXML, WorkspaceXML})
    End Sub

    Friend Sub ReadBloclyToolboxFromFile()
        If IO.File.Exists(BlocklyFilesPath + "XML\blockyToolbox.xml") Then
            ToolboxXML = System.IO.File.ReadAllText(BlocklyFilesPath + "XML\blockyToolbox.xml")
        End If
    End Sub

    Public Sub ExecuteScriptFunction(ByVal FunctionName As String)
        Form1.WV2.ExecuteScriptAsync(FunctionName)
    End Sub

    Public Async Sub ExecuteScriptFunctionAsync(ByVal FunctionName As String)
        Await Form1.WV2.ExecuteScriptAsync(FunctionName)
    End Sub

    'Friend Async Sub HighlightBlock(id As String)
    '    Dim s(0) As String
    '    s(0) = id
    '    Await ExecuteScriptFunctionAsync("highlightBlock", s)
    'End Sub

    Public Async Function ExecuteScriptFunctionAsync(ByVal FunctionName As String,
                                                     ByVal Parameters() As Object) _
                                                     As Threading.Tasks.Task(Of String)
        Dim script As String = FunctionName + "("
        Dim i As Integer = 0
        Do While i < Parameters.Length
            script += JsonConvert.SerializeObject(Parameters(i))
            If i < (Parameters.Length - 1) Then
                script += ", "
            End If
            i += 1
        Loop
        script += ");"

        Dim result As Object = Await Form1.WV2.ExecuteScriptAsync(script)
        result = CType(JsonConvert.DeserializeObject(result.ToString), String)
        Return CStr(result)
    End Function

    ' ===================================================================================================
    '  GET BLOCKLY CODE
    ' ===================================================================================================
    '  Wait until async code is ready 
    ' ===================================================================================================
    Friend Function GetBlocklyCode() As String
        For i As Int32 = 1 To 100 ' <-- timeout 
            GetBlocklyCode2()
            Application.DoEvents()
            Threading.Thread.Sleep(10)
            If BlocklyCode IsNot Nothing AndAlso BlocklyCode <> "" Then
                Return BlocklyCode
            End If
        Next
        Return ""
    End Function

    Private BlocklyCode As String
    Friend Async Sub GetBlocklyCode2()
        BlocklyCode = ""
        Dim result As Object = Await Form1.WV2.ExecuteScriptAsync("getBlocklyCode()")
        BlocklyCode = CType(JsonConvert.DeserializeObject(result.ToString), String)
    End Sub

    Friend Sub ShowBlocklyCode()
        Dim s As String = ""
        s += vbCrLf
        s += "=================================================== " + vbCrLf
        s += "  BLOCKLY CODE " + vbCrLf
        s += "=================================================== " + vbCrLf
        s += GetBlocklyCode()
        s += "=================================================== " + vbCrLf
        AddTextToOutputBox(s)
    End Sub

    Friend Sub ShowBlocklyWorkspace()
        GetBlocklyWorkspace()
        Dim s As String = ""
        s += vbCrLf
        s += "=================================================== " + vbCrLf
        s += "  BLOCKLY WORKSPACE " + vbCrLf
        s += "=================================================== " + vbCrLf
        s += WorkspaceXML + vbCrLf
        s += "=================================================== " + vbCrLf
        AddTextToOutputBox(s)
    End Sub

    ' ===================================================================================================
    '  GET BLOCKLY WORKSPACE
    ' ===================================================================================================
    '  With a workaround to get the updated workspace (otherwise last editing are loosed) 
    ' ===================================================================================================
    Friend Sub GetBlocklyWorkspace()
        For i As Int32 = 1 To 100 ' <-- timeout
            GetBlocklyWorkspace2()
            Application.DoEvents()
            Threading.Thread.Sleep(10)
            If WorkspaceXML IsNot Nothing AndAlso
                WorkspaceXML.StartsWith("<xml xmlns=") AndAlso
                WorkspaceXML.EndsWith("</xml>") Then
                'Form1.Text = i.ToString
                'Console.Beep(1500, 25)
                Click2()
                Return
            End If
            Click1c()
        Next
        Form1.BreakRunningLoop = True
    End Sub
    Private Async Sub GetBlocklyWorkspace2()
        WorkspaceXML = ""
        Dim s As String
        s = Await Form1.WV2.ExecuteScriptAsync("getWorkspaceAsText()")
        s = CType(JsonConvert.DeserializeObject(s), String)
        WorkspaceXML = s
    End Sub

    ' ===================================================================================================
    '  GET BLOCKLY TOOLBOX
    ' ===================================================================================================
    Friend Async Sub GetBlocklyToolbox()
        Dim s As String
        s = Await Form1.WV2.ExecuteScriptAsync("getToolboxAsText()")
        s = CType(JsonConvert.DeserializeObject(s), String)
        MessageBox.Show(s + vbCrLf + vbCrLf + s.Length.ToString)
    End Sub

End Module
