﻿Module DecodeCommands

    ' ===================================================================================================
    '  DECODE BLOCKLY COMMANDS - INPUT
    ' ===================================================================================================
    Friend Function DecodeCommands_INPUT(ByVal s1 As String) As String
        Dim s2 As String
        s2 = ReplaceMultipleSpacesAndTabAndTrim(s1)
        Dim sa() As String = s2.Split(" "c)
        ' -------------------------------------------------------------------
        Select Case sa(0).ToLower
            Case "read_slot" : Return Slots.ReadSlot(Val_Int(sa(1))).ToString(GCI)
            Case "read_slottext" : Return StringSlots.ReadString(Val_Int(sa(1))).ToString(GCI)
                '
            Case "com_status" : Return COM_GetStatus()
            Case "com_received" : Return COM_GetReceivedBuffer()
            Case "com_receiveuntil"
                sa(1) = sa(1).Replace("\r", vbCr)
                sa(1) = sa(1).Replace("\n", vbLf)
                Return COM_GetReceivedUntil(sa(1))
            Case "com_receivenchars" : Return COM_GetReceivedNBytes(Val_Int(sa(1)))
            Case "com_portnames"
                Dim sb() As String = COM_GetPortNames()
                Dim names As String = ""
                For Each s As String In sb
                    names += s + " "
                Next
                Return names.Trim
            Case "com_getdsr" : Return If(COM_GetDSR(), "ON", "OFF")
            Case "com_getcts" : Return If(COM_GetCTS(), "ON", "OFF")
                '
            Case "tts_voices" : Return TTS_GetInstalledVoices()
                '
            Case Else
                Return ""
        End Select
    End Function


    ' ===================================================================================================
    '  DECODE BLOCKLY COMMANDS - OUTPUT
    ' ===================================================================================================
    Friend Sub DecodeCommands_OUTPUT(s1 As String)
        Dim s2 As String = s1.ToLower
        s2 = ReplaceMultipleSpacesAndTabAndTrim(s2)
        Dim sa() As String = s2.Split(" "c)
        ' ------------------------------------------------------ PARSE SOME COMMANDS
        Select Case sa(0)
            Case "print", "tts_speak", "tts_voice", "com_sendtext"
                If sa.Length > 2 Then
                    For i As Int32 = 2 To sa.Length - 1
                        sa(1) += " " + sa(i)
                    Next
                    ReDim Preserve sa(1)
                End If
            Case "write_slottext"
                If sa(0) = "write_slottext" AndAlso sa.Length > 3 Then
                    For i As Int32 = 3 To sa.Length - 1
                        sa(2) += " " + sa(i)
                    Next
                    ReDim Preserve sa(2)
                End If
        End Select
        ' ------------------------------------------------------ DECODE
        Select Case sa.Length
            Case 1
                Select Case sa(0)
                    Case "program_start"
                        '
                    Case "program_changed"
                        If GetRunningSeconds() > 10 Or Form1.BlocksCounter > 1 Then
                            Form1.StopRunning()
                        End If
                    Case "program_end" : Form1.StopRunning()
                        '
                    Case "open_apps" : OpenAllThereminicApps()
                    Case "close_apps" : CloseAllOpenedProcesses()
                    Case "close_all" : Form1.FlagCloseAll = True
                        '
                    Case "tts_stop" : TTS_Stop()
                    Case "tts_pause" : TTS_Pause()
                    Case "tts_resume" : TTS_Resume()
                    Case "tts_wait"  ' Waiting is done in the JS function
                        '
                    Case "com_close" : COM_Close()
                    Case "com_discardinbuffer" : COM_DiscardInBuffer()
                    Case "com_discardoutbuffer" : COM_DiscardOutBuffer()
                    Case "com_enabledtr" : COM_EnableDTR()
                    Case "com_enablerts" : COM_EnableRTS()
                    Case "com_disabledtr" : COM_DisableDTR()
                    Case "com_disablerts" : COM_DisableRTS()
                        '
                    Case "enable_all_buttons" : Form1.EnableAllButtons() : Form1.GroupBox1.Refresh()
                    Case "disable_all_buttons" : Form1.DisableAllButtons() : Form1.GroupBox1.Refresh()
                        '
                    Case "open_textbox" : Form1.OpenTextBox()
                    Case "close_textbox" : Form1.CloseTextBox()
                    Case "clear_textbox" : Form1.ClearTextBox()
                        '
                    Case Else
                        UnrecognizedCommand(s1)
                End Select
            Case 2
                Select Case sa(0)
                    Case "print"
                    Case "write_slottext" : StringSlots.WriteString(Val_Int(sa(1)), "")
                        '
                    Case "open_app"
                        Dim fname As String = sa(1).Trim.Replace("""", "")
                        OpenAppByFilename(fname)
                    Case "close_app"
                        Dim fname As String = sa(1).Trim.Replace("""", "")
                        CloseAppByFilename(fname)
                        '
                    Case "tts_volume" : TTS_SetVolume(Val_Int(sa(1)))
                    Case "tts_speed" : TTS_SetSpeed(Val_Int(sa(1)))
                    Case "tts_voice" : TTS_SelectVoice(sa(1))
                    Case "tts_speak" : TTS_SpeakText(sa(1))
                        '
                    Case "com_sendtext" : COM_WriteString(ReplaceEscapeCharsWithCRLF(sa(1)))
                        '
                    Case "enable_button"
                        Dim n As Int32 = Val_Int(sa(1)) - 1
                        Form1.EnableButton(n)
                        Form1.Buttons(n).Refresh()
                    Case "disable_button"
                        Dim n As Int32 = Val_Int(sa(1)) - 1
                        Form1.DisableButton(n)
                        Form1.Buttons(n).Refresh()
                        '
                    Case Else
                        UnrecognizedCommand(s1)
                End Select
            Case 3
                Select Case sa(0)
                    Case "beep" : Console.Beep(Val_Int(sa(1)), Val_Int(sa(2)))
                    Case "write_slot" : Slots.WriteSlot(Val_Int(sa(1)), Val_Single(sa(2)))
                    Case "write_slottext" : StringSlots.WriteString(Val_Int(sa(1)), sa(2))
                        '
                    Case "com_open" : COM_Open(sa(1), Val_Int(sa(2)))
                    Case "com_sendtextdouble" : COM_WriteString(ReplaceEscapeCharsWithCRLF(sa(1)) + ReplaceEscapeCharsWithCRLF(sa(2)))
                        '
                    Case "button_colour" : Form1.SetButtonColor(Val_Int(sa(1)), sa(2))
                    Case "button_slot" : Form1.SetButtonSlot(Val_Int(sa(1)), Val_Int(sa(2)))
                    Case "button_text" : Form1.SetButtonText(Val_Int(sa(1)), sa(2))
                        '
                    Case Else
                        UnrecognizedCommand(s1)
                End Select
            Case 5
                Select Case sa(0)
                    Case "button_colour_flashing" : Form1.SetButtonColorsFlashing(Val_Int(sa(1)), sa(2), sa(3), Val_Int(sa(4)))
                        '
                    Case Else
                        UnrecognizedCommand(s1)
                End Select
            Case Else
                        UnrecognizedCommand(s1)
                End Select
    End Sub

    Private Sub UnrecognizedCommand(s As String)
        AddTextToOutputBox("UNRECOGNIZED " + s)
    End Sub

    Private Function ReplaceEscapeCharsWithCRLF(ByVal str As String) As String
        str = str.Replace("\r", vbCr)
        str = str.Replace("\n", vbLf)
        Return str
    End Function

    ' ===================================================================================================
    '  ADD TEXT TO OUTPUT BOX
    ' ===================================================================================================
    Friend ReceivedTextChanged As Boolean
    Friend ReceivedText As String
    Friend Sub AddTextToOutputBox(txt As String)
        If txt.ToLower.StartsWith("print ") Then
            If Not Form1.Chk_ShowDebugMessages.Checked Then
                txt = txt.Substring(6)
            End If
        Else
            If Not Form1.Chk_ShowDebugMessages.Checked Then Return
            txt = "DEBUG: " + txt
        End If
        ' ---------------------------------------------------------------------
        ReceivedText = Strings.Right(ReceivedText, 9000) + txt + vbCrLf
        ReceivedTextChanged = True
    End Sub

    ' ====================================================
    '  FAST APPEND To Long STRINGS With String-BUILDER
    ' ====================================================
    'Dim sb As StringBuilder = New StringBuilder
    'sb.Append(StrDup(Indent, " ") + str(n) + vbCrLf)
    'Return sb.ToString


    ' ===================================================================================================
    '  RUNNING TIME
    ' ===================================================================================================
    Private ReadOnly SW As New Stopwatch
    Friend Sub StartRunningTime()
        SW.Reset()
        SW.Start()
    End Sub
    Friend Function GetRunningSeconds() As Double
        Return SW.Elapsed.TotalSeconds
    End Function
    Friend Function GetRunningMilliSeconds() As Double
        Return SW.Elapsed.TotalMilliseconds
    End Function

End Module
